{% extends "ai_kit_auth/body_base.html" %}

{% block title %}Passwort zurücksetzen{% endblock %}

{% block text %}
    <p>Guten Tag {{ user.username }},</p>
    <p>Klicken Sie auf folgenden Link, um Ihr Passwort zurück zu setzen.</p>
{% endblock %}

{% block link %}
<a href="{{ url }}" target="_blank">PASSWORD ZURÜCKSETZEN</a>
{% endblock %}
