{% extends "ai_kit_auth/body_base.html" %}

{% block title %}Aktivierung des Accounts{% endblock %}

{% block text %}
    <p>Guten Tag {{ user.username }},</p>
    <p>Sie sind nur noch einen Klick davon entfernt, Ihren Account zu benutzen.</p>
{% endblock %}

{% block link %}
<a href="{{ url }}" target="_blank">KONTO AKTIVIEREN</a>
{% endblock %}