{% extends "ai_kit_auth/body_base.html" %}

{% block title %}Ihr Account wurde angelegt{% endblock %}

{% block text %}
    <p>Guten Tag {{ user.username }},</p>
    <p>Es wurde ein Nutzeraccount für Sie angelegt.</p>
    <p>Bitte folgen Sie folgendem Link, um Ihr Passwort zu setzen</p>
{% endblock %}

{% block link %}
<a href="{{ url }}" target="_blank">PASSWORD ZURÜCKSETZEN</a>
{% endblock %}}